mob/proc/guess()
	set category=null
	set desc="Confirm the current guess."

	//Make sure the game has started
	if (!gamestarted)
		src << "The game has not yet started."
		return

	//Confirmation alert
	if (alert(src,"Do you want to confirm your current guess?","Confirm guess","Confirm","No")=="No") return

	//Count up pegs in the sequence
	var/theguess[0]
	var/pegs[0]
	for(var/pegx=14-codelength to 13)
		var/turf/T=locate(pegx,1,1)
		var/obj/peg/P=locate() in T
		if (!P)
			//No peg entered; use same as last one
			var/obj/peg/lastpeg=locate() in locate(pegx,currentrow-1,1)
			if (!lastpeg)
				//There is no last peg!
				alert(src,"You must fill in all the holes.\n\nOn later turns, you \
				may omit some holes to use your previous guesses in them; but as this is the first turn, \
				there are no previous guesses!","Holes not filled in")
				return
			else
				//Duplicate that peg
				P=new(locate(pegx,1,1), lastpeg.colour)
		pegs+=P
		theguess+=P.colour

	//Move the pegs
	sleep()
	for (var/obj/peg/P in pegs) P.Move(locate(P.x,currentrow,1))

	//Tell them what their guess was
	src << "You guessed: \..."
	for (var/X in theguess) src << "<font color=[colours[X]]>[X]</font> \..."
	src << ""

	//Work out how much of their guess was right
	var/results[]=scoreguess(theguess)

	//Set up hit/miss markers
	var/column=14
	for (var/X=1 to results["hit"])
		column++
		new/obj/marker/hit(locate(column,currentrow,1))
	for (var/X=1 to results["miss"])
		column++
		new/obj/marker/miss(locate(column,currentrow,1))

	//If they won, end the game
	if (results["hit"]==codelength)
		gameover(1)
		return

	//Tell them the result
	if (results["hit"]) src << "Correct colour, correct location: [results["hit"]] peg\s"
	if (results["miss"]) src << "Correct colour, wrong location: [results["miss"]] peg\s"
	if (!results["hit"] && !results["miss"]) src << "No correct pegs."

	//Move the row counter down
	currentrow++

	//Did they lose?
	if (currentrow>20)
		gameover(0)
		return

proc/scoreguess(list/guess)
	var/hit=0
	var/miss=0
	var/codecopy[]=thecode.Copy()
	var/guesscopy[]=guess.Copy()
	//Find hits
	for (var/X=1 to length(guesscopy))
		if (!guesscopy[X]) continue
		if (guesscopy[X]==codecopy[X])
			hit++
			codecopy[X]=""
			guesscopy[X]=""
	//Find misses
	for (var/X=1 to length(guesscopy))
		if (!guesscopy[X]) continue
		var/location=codecopy.Find(guesscopy[X])
		if (location)
			miss++
			codecopy[location]=""
			guesscopy[X]=""
	return list("hit"=hit,"miss"=miss)

proc/sequence(thenum as num)
	var/thelist[0]
	for (var/X=1 to thenum) thelist+=X
	return thelist